@@ -52,6 +52,7 @@
 #include <KIO/FileUndoManager>
 #include <KIO/JobUiDelegate>
 #include <KIO/JobUiDelegateFactory>
+#include <KIO/OpenFileManagerWindowJob>
 #include <KIO/OpenUrlJob>
 #include <KIO/Paste>
 #include <KIO/PasteJob>
@@ -59,6 +60,7 @@
 #include <KIO/RestoreJob>
 #include <KIO/StatJob>
 #include <KLocalizedString>
+#include <KNotification>
 #include <KNotificationJobUiDelegate>
 #include <KPropertiesDialog>
 #include <KProtocolInfo>
@@ -174,8 +176,6 @@ FolderModel::FolderModel(QObject *parent)
 {
     connect(DragTracker::self(), &DragTracker::dragInProgressChanged, this, &FolderModel::draggingChanged);
     connect(DragTracker::self(), &DragTracker::dragInProgressChanged, this, &FolderModel::dragInProgressAnywhereChanged);
-    // needed to pass the job around with qml
-    qmlRegisterAnonymousType<KIO::DropJob>("org.kde.private.desktopcontainment.folder", 0);
     DirLister *dirLister = new DirLister(this);
     dirLister->setDelayedMimeTypes(true);
     dirLister->setAutoErrorHandlingEnabled(false);
@@ -184,14 +184,12 @@ FolderModel::FolderModel(QObject *parent)
 
     connect(dirLister, &KCoreDirLister::started, this, std::bind(&FolderModel::setStatus, this, Status::Listing));
 
-    void (KCoreDirLister::*myCompletedSignal)() = &KCoreDirLister::completed;
-    QObject::connect(dirLister, myCompletedSignal, this, [this] {
+    QObject::connect(dirLister, &KCoreDirLister::completed, this, [this] {
         setStatus(Status::Ready);
         Q_EMIT listingCompleted();
     });
 
-    void (KCoreDirLister::*myCanceledSignal)() = &KCoreDirLister::canceled;
-    QObject::connect(dirLister, myCanceledSignal, this, [this] {
+    QObject::connect(dirLister, &KCoreDirLister::canceled, this, [this] {
         setStatus(Status::Canceled);
         Q_EMIT listingCanceled();
     });
@@ -282,18 +280,18 @@ QHash<int, QByteArray> FolderModel::roleNames() const
 QHash<int, QByteArray> FolderModel::staticRoleNames()
 {
     QHash<int, QByteArray> roleNames;
-    roleNames[Qt::DisplayRole] = "display";
-    roleNames[Qt::DecorationRole] = "decoration";
-    roleNames[BlankRole] = "blank";
-    roleNames[SelectedRole] = "selected";
-    roleNames[IsDirRole] = "isDir";
-    roleNames[IsLinkRole] = "isLink";
-    roleNames[IsHiddenRole] = "isHidden";
-    roleNames[UrlRole] = "url";
-    roleNames[LinkDestinationUrl] = "linkDestinationUrl";
-    roleNames[SizeRole] = "size";
-    roleNames[TypeRole] = "type";
-    roleNames[FileNameWrappedRole] = "displayWrapped";
+    roleNames[Qt::DisplayRole] = QByteArrayLiteral("display");
+    roleNames[Qt::DecorationRole] = QByteArrayLiteral("decoration");
+    roleNames[BlankRole] = QByteArrayLiteral("blank");
+    roleNames[SelectedRole] = QByteArrayLiteral("selected");
+    roleNames[IsDirRole] = QByteArrayLiteral("isDir");
+    roleNames[IsLinkRole] = QByteArrayLiteral("isLink");
+    roleNames[IsHiddenRole] = QByteArrayLiteral("isHidden");
+    roleNames[UrlRole] = QByteArrayLiteral("url");
+    roleNames[LinkDestinationUrl] = QByteArrayLiteral("linkDestinationUrl");
+    roleNames[SizeRole] = QByteArrayLiteral("size");
+    roleNames[TypeRole] = QByteArrayLiteral("type");
+    roleNames[FileNameWrappedRole] = QByteArrayLiteral("displayWrapped");
 
     return roleNames;
 }
@@ -744,18 +742,23 @@ void FolderModel::setFilterMimeTypes(const QStringList &mimeList)
 
 void FolderModel::setScreen(int screen)
 {
-    m_screenUsed = (screen != -1);
+    bool screenUsed = (screen != -1);
 
-    if (!m_screenUsed || m_screen == screen)
-        return;
-
-    m_screen = screen;
-    if (m_usedByContainment && !m_screenMapper->sharedDesktops()) {
-        m_screenMapper->addScreen(screen, m_currentActivity, resolvedUrl());
+    if (screenUsed && m_screen != screen) {
+        m_screen = screen;
+        if (m_usedByContainment && !m_screenMapper->sharedDesktops()) {
+            m_screenMapper->addScreen(screen, m_currentActivity, resolvedUrl());
+        }
     }
+    m_screenUsed = screenUsed;
     Q_EMIT screenChanged();
 }
 
+bool FolderModel::screenUsed()
+{
+    return m_screenUsed;
+}
+
 KFileItem FolderModel::rootItem() const
 {
     return m_dirModel->dirLister()->rootItem();
@@ -844,6 +847,38 @@ void FolderModel::runSelected()
     fileItemActions.runPreferredApplications(items);
 }
 
+void FolderModel::showTarget()
+{
+    const QModelIndexList indexes = m_selectionModel->selectedIndexes();
+
+    if (indexes.isEmpty() || indexes.count() != 1) {
+        return;
+    }
+
+    const KFileItem link = itemForIndex(indexes.first());
+    const QUrl destinationUrl = QUrl::fromLocalFile(link.linkDest());
+
+    auto job = KIO::stat(destinationUrl, KIO::StatJob::SourceSide, KIO::StatNoDetails);
+
+    connect(job, &KJob::finished, this, [link, destinationUrl](KJob *job) {
+        KIO::StatJob *statJob = static_cast<KIO::StatJob *>(job);
+
+        if (statJob->error()) {
+            KNotification::event(
+                KNotification::Error,
+                i18nc("@title:notifications Here 'link' refers to a symbolic link to another file or folder", "Link Target Not Found"),
+                xi18nc("@info Body text of a system notification",
+                       "<filename>%1</filename> points to <filename>%2</filename>, but that could not be found. It may have been moved or deleted.",
+                       link.name(),
+                       link.linkDest()),
+                QStringLiteral("dialog-error"),
+                KNotification::DefaultEvent);
+        } else {
+            KIO::highlightInFileManager({destinationUrl});
+        }
+    });
+}
+
 void FolderModel::rename(int row, const QString &name)
 {
     if (row < 0) {
@@ -903,6 +938,7 @@ void FolderModel::setSelected(int row)
     }
 
     m_selectionModel->select(index(row, 0), QItemSelectionModel::Select);
+    Q_EMIT selectionDone();
 }
 
 void FolderModel::toggleSelected(int row)
@@ -912,6 +948,7 @@ void FolderModel::toggleSelected(int row)
     }
 
     m_selectionModel->select(index(row, 0), QItemSelectionModel::Toggle);
+    Q_EMIT selectionDone();
 }
 
 void FolderModel::setRangeSelected(int anchor, int to)
@@ -922,6 +959,7 @@ void FolderModel::setRangeSelected(int anchor, int to)
 
     QItemSelection selection(index(anchor, 0), index(to, 0));
     m_selectionModel->select(selection, QItemSelectionModel::ClearAndSelect);
+    Q_EMIT selectionDone();
 }
 
 void FolderModel::updateSelection(const QVariantList &rows, bool toggle)
@@ -948,6 +986,8 @@ void FolderModel::updateSelection(const QVariantList &rows, bool toggle)
     } else {
         m_selectionModel->select(newSelection, QItemSelectionModel::ClearAndSelect);
     }
+    // We do not emit selectionDone here since updateSelection is called on every movement
+    // and we want to call it after selection is done for performance reasons
 }
 
 void FolderModel::clearSelection()
@@ -1695,7 +1735,14 @@ void FolderModel::createActions()
     QAction *actOpen = new QAction(QIcon::fromTheme(QStringLiteral("window-new")), i18n("&Open"), this);
     connect(actOpen, &QAction::triggered, this, &FolderModel::runSelected);
 
+    QAction *showTarget = m_actionCollection.addAction(QStringLiteral("showTarget"));
+    showTarget->setText(i18nc("@action:inmenu open file manager showing the target file or folder that this link points to", "Show Target"));
+    showTarget->setIcon(QIcon::fromTheme(QStringLiteral("document-open-folder")));
+    showTarget->setEnabled(false);
+    connect(showTarget, &QAction::triggered, this, &FolderModel::showTarget);
+
     m_actionCollection.addAction(QStringLiteral("open"), actOpen);
+    m_actionCollection.addAction(QStringLiteral("showTarget"), showTarget);
     m_actionCollection.addAction(QStringLiteral("cut"), cut);
     m_actionCollection.addAction(QStringLiteral("undo"), undo);
     m_actionCollection.addAction(QStringLiteral("copy"), copy);
@@ -1766,6 +1813,10 @@ void FolderModel::updateActions()
         }
     }
 
+    if (QAction *showTarget = m_actionCollection.action(QStringLiteral("showTarget"))) {
+        showTarget->setEnabled(items.count() == 1 && items.at(0).isLink());
+    }
+
     if (m_newMenu) {
         m_newMenu->checkUpToDate();
         m_newMenu->setWorkingDirectory(m_dirModel->dirLister()->url());
@@ -1876,6 +1927,14 @@ void FolderModel::openContextMenu(QQuickItem *visualParent, Qt::KeyboardModifier
         m_fileItemActions->setItemListProperties(itemProperties);
         m_fileItemActions->insertOpenWithActionsTo(nullptr, menu, QStringList());
         menu->addSeparator();
+
+        // Add "Show Target" action only if the file *has* a target
+        QAction *showTargetAction = m_actionCollection.action(QStringLiteral("showTarget"));
+        if (showTargetAction->isEnabled()) {
+            menu->addAction(showTargetAction);
+            menu->addSeparator();
+        }
+
         menu->addAction(m_actionCollection.action(QStringLiteral("cut")));
         menu->addAction(m_actionCollection.action(QStringLiteral("copy")));
         if (urls.length() == 1) {
@@ -2089,6 +2148,7 @@ void FolderModel::setApplet(Plasma::Applet *applet)
                 }
                 setScreen(containment->screen());
                 connect(containment, &Plasma::Containment::screenChanged, this, &FolderModel::setScreen);
+                connect(containment, &Plasma::Containment::screenGeometryChanged, this, &FolderModel::screenGeometryChanged);
             }
         }
 
@@ -2096,6 +2156,17 @@ void FolderModel::setApplet(Plasma::Applet *applet)
     }
 }
 
+QRectF FolderModel::screenGeometry()
+{
+    if (m_applet) {
+        Plasma::Containment *containment = m_applet->containment();
+        if (containment) {
+            return containment->screenGeometry();
+        }
+    }
+    return QRectF();
+}
+
 bool FolderModel::showHiddenFiles() const
 {
     return m_showHiddenFiles;
@@ -2207,3 +2278,5 @@ bool FolderModel::isDeleteCommandShown()
     KConfigGroup cg(KSharedConfig::openConfig(), QStringLiteral("KDE"));
     return cg.readEntry("ShowDeleteCommand", false);
 }
+
+#include "moc_foldermodel.cpp"
